@@ -1,6 +1,6 @@
 #!perl
 # readme_md.pl: Make README.md from a Perl file.
-# Part of Data::Hopen.
+# Part of Class::Tiny::ConstrainedAccessor.
 
 use 5.014;
 use strict;
@@ -49,7 +49,6 @@ open my $fh, '<', \$parsed;
 # Filter and tweak the POD
 my $saw_name = 0;
 my $tweak_name = ($format eq 'md');
-my $force_conventions = ($format eq 'md');
 my $output = '';
 
 while(my $line = <$fh>) {
@@ -70,11 +69,9 @@ while(my $line = <$fh>) {
 
     next if $line =~ /SYNOPSIS/;    # Don't need this header.
 
-    # Skip the internals
-    $output .= $line if $line =~ /AUTHOR/;
-    next if ($line =~ /SUBROUTINES/)..($line =~ /AUTHOR/);
-
-    $line =~ s{https://metacpan.org/pod/Data::Hopen::Conventions}{https://metacpan.org/pod/release/CXW/Build-Hopen-0.000006-TRIAL/lib/Build/Hopen/Conventions.pod} if $force_conventions;
+##    # Skip the internals
+##    $output .= $line if $line =~ /AUTHOR/;
+##    next if ($line =~ /SUBROUTINES/)..($line =~ /AUTHOR/);
 
     $output .= $line;   # Copy everything that's left.
 }
